#!/usr/bin/env python
# -*- coding: utf-8 -*-

SERIAL_TYPE = (
    'aastory',
    'bgif2',
    'biquge',
    'danmei123',
    'dzxsw',
    'feizw',
    'haxtxt',
    'icnsp',
    'klxsw',
    'lwxs',
    'lwxs520',
    'lwxsw',
    'piaotian',
    'piaotiancc',
    'ranwen',
    'shu69',
    'shushu8',
    'sto',
    'ttshuba',
    'ttzw',
    'ttzw5',
    'uks5',
    'wdxs',
    'xs365',
    'yfzww',
    'yq33',
    'zhaishu8',
)

ARTICLE_TYPE = (
    'cool18',
    'doubangroup',
    'park6',
    'sdragon',
    'sis',
)

# Configuration for free ip in CERNET
CERNET_USE_PROXIES = (
    'cool18',
    'danmei123',
    'dzxsw',
    'haxtxt',
    'klxsw',
    'lwxs520',
    'lwxs',
    'lwxsw',
    'park6',
    'piaotian',
    'piaotiancc',
    'ranwen',
    'sdragon',
    'shu69',
    'shushu8',
    'sis',
    'sto',
    'ttshuba',
    'ttzw',
    'uks5',
    'wdxs',
    'xs365',
    'yq33',
    'zhaishu8',
)

DEFAULT_NOT_OVERWRITE = (
    'cool18',
    'park6',
)

AUTO_MARK_FINISH = (
    'sto',
)
